@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2005 Joe Walnes.
- * Copyright (C) 2006, 2007, 2008 XStream Committers.
+ * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -11,31 +11,37 @@
  */
 package com.thoughtworks.xstream.converters.javabean;
 
-import com.thoughtworks.xstream.alias.ClassMapper;
+import java.util.HashSet;
+import java.util.Set;
+
 import com.thoughtworks.xstream.converters.ConversionException;
 import com.thoughtworks.xstream.converters.Converter;
 import com.thoughtworks.xstream.converters.MarshallingContext;
 import com.thoughtworks.xstream.converters.UnmarshallingContext;
+import com.thoughtworks.xstream.converters.reflection.MissingFieldException;
+import com.thoughtworks.xstream.core.util.FastField;
 import com.thoughtworks.xstream.io.HierarchicalStreamReader;
 import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
 import com.thoughtworks.xstream.io.ExtendedHierarchicalStreamWriterHelper;
 import com.thoughtworks.xstream.mapper.Mapper;
 
 /**
- * Can convert any bean with a public default constructor. BeanInfo are not
- * taken into consideration, this class looks for bean patterns for simple
- * properties
+ * Can convert any bean with a public default constructor. The {@link BeanProvider} used as
+ * default is based on {@link java.beans.BeanInfo}. Indexed properties are currently not supported.
  */
 public class JavaBeanConverter implements Converter {
 
     /*
      * TODO:
      *  - support indexed properties
+     *  - support attributes (XSTR-620)
+     *  - support local converters (XSTR-601)
+     *  Problem: Mappers take definitions based on reflection, they don't know about bean info
      */
-    private Mapper mapper;
-    private BeanProvider beanProvider;
+    protected final Mapper mapper;
+    protected final JavaBeanProvider beanProvider;
     /**
-     * @deprecated since 1.2, no necessity for field anymore.
+     * @deprecated As of 1.3, no necessity for field anymore.
      */
     private String classAttributeIdentifier;
 
@@ -43,7 +49,7 @@ public class JavaBeanConverter implements Converter {
         this(mapper, new BeanProvider());
     }
 
-    public JavaBeanConverter(Mapper mapper, BeanProvider beanProvider) {
+    public JavaBeanConverter(Mapper mapper, JavaBeanProvider beanProvider) {
         this.mapper = mapper;
         this.beanProvider = beanProvider;
     }
@@ -56,13 +62,6 @@ public class JavaBeanConverter implements Converter {
         this.classAttributeIdentifier = classAttributeIdentifier;
     }
 
-    /**
-     * @deprecated As of 1.2, use {@link #JavaBeanConverter(Mapper)} and {@link com.thoughtworks.xstream.XStream#aliasAttribute(String, String)}
-     */
-    public JavaBeanConverter(ClassMapper classMapper, String classAttributeIdentifier) {
-        this((Mapper)classMapper, classAttributeIdentifier);
-    }
-
     /**
      * Only checks for the availability of a public default constructor.
      * If you need stricter checks, subclass JavaBeanConverter
@@ -72,10 +71,14 @@ public class JavaBeanConverter implements Converter {
     }
 
     public void marshal(final Object source, final HierarchicalStreamWriter writer, final MarshallingContext context) {
-        final String classAttributeName = classAttributeIdentifier != null ? classAttributeIdentifier : mapper.attributeForAlias("class");
-        beanProvider.visitSerializableProperties(source, new BeanProvider.Visitor() {
+        final String classAttributeName = classAttributeIdentifier != null ? classAttributeIdentifier : mapper.aliasForSystemAttribute("class");
+        beanProvider.visitSerializableProperties(source, new JavaBeanProvider.Visitor() {
+            public boolean shouldVisit(String name, Class definedIn) {
+                return mapper.shouldSerializeMember(definedIn, name);
+            }
+            
             public void visit(String propertyName, Class fieldType, Class definedIn, Object newObj) {
-                if (newObj != null && mapper.shouldSerializeMember(definedIn, propertyName)) {
+                if (newObj != null) {
                     writeField(propertyName, fieldType, newObj, definedIn);
                 }
             }
@@ -84,37 +87,46 @@ public class JavaBeanConverter implements Converter {
                 String serializedMember = mapper.serializedMember(source.getClass(), propertyName);
 				ExtendedHierarchicalStreamWriterHelper.startNode(writer, serializedMember, fieldType);
                 Class actualType = newObj.getClass();
-
                 Class defaultType = mapper.defaultImplementationOf(fieldType);
-                if (!actualType.equals(defaultType)) {
+                if (!actualType.equals(defaultType) && classAttributeName != null) {
                     writer.addAttribute(classAttributeName, mapper.serializedClass(actualType));
                 }
                 context.convertAnother(newObj);
 
                 writer.endNode();
             }
-
         });
     }
 
     public Object unmarshal(final HierarchicalStreamReader reader, final UnmarshallingContext context) {
         final Object result = instantiateNewInstance(context);
+        final Set seenProperties = new HashSet() {
+            public boolean add(Object e) {
+                if (!super.add(e)) {
+                    throw new DuplicatePropertyException(((FastField)e).getName());
+                }
+                return true;
+            }
+        };
 
+        Class resultType = result.getClass();
         while (reader.hasMoreChildren()) {
             reader.moveDown();
 
-            String propertyName = mapper.realMember(result.getClass(), reader.getNodeName());
+            String propertyName = mapper.realMember(resultType, reader.getNodeName());
 
-            boolean propertyExistsInClass = beanProvider.propertyDefinedInClass(propertyName, result.getClass());
-
-            if (propertyExistsInClass) {
-                Class type = determineType(reader, result, propertyName);
-                Object value = context.convertAnother(result, type);
-                beanProvider.writeProperty(result, propertyName, value);
-            } else if (mapper.shouldSerializeMember(result.getClass(), propertyName)) {
-                throw new ConversionException("Property '" + propertyName + "' not defined in class " + result.getClass().getName());
+            if (mapper.shouldSerializeMember(resultType, propertyName)) {
+                boolean propertyExistsInClass = beanProvider.propertyDefinedInClass(propertyName, resultType);
+    
+                if (propertyExistsInClass) {
+                    Class type = determineType(reader, result, propertyName);
+                    Object value = context.convertAnother(result, type);
+                    beanProvider.writeProperty(result, propertyName, value);
+                    seenProperties.add(new FastField(resultType, propertyName));
+                } else {
+                    throw new MissingFieldException(resultType.getName(), propertyName);
+                }
             }
-
             reader.moveUp();
         }
 
@@ -130,8 +142,8 @@ public class JavaBeanConverter implements Converter {
     }
 
     private Class determineType(HierarchicalStreamReader reader, Object result, String fieldName) {
-        final String classAttributeName = classAttributeIdentifier != null ? classAttributeIdentifier : mapper.attributeForAlias("class");
-        String classAttribute = reader.getAttribute(classAttributeName);
+        final String classAttributeName = classAttributeIdentifier != null ? classAttributeIdentifier : mapper.aliasForSystemAttribute("class");
+        String classAttribute = classAttributeName == null ? null : reader.getAttribute(classAttributeName);
         if (classAttribute != null) {
             return mapper.realClass(classAttribute);
         } else {
@@ -140,11 +152,24 @@ public class JavaBeanConverter implements Converter {
     }
 
     /**
-     * @deprecated since 1.3
+     * @deprecated As of 1.3
      */
     public static class DuplicateFieldException extends ConversionException {
         public DuplicateFieldException(String msg) {
             super(msg);
         }
     }
+
+    /**
+     * Exception to indicate double processing of a property to avoid silent clobbering.
+     * 
+     * @author J&ouml;rg Schaible
+     * @since 1.4.2
+     */
+    public static class DuplicatePropertyException extends ConversionException {
+        public DuplicatePropertyException(String msg) {
+            super("Duplicate property " + msg);
+            add("property", msg);
+        }
+    }
 }
\ No newline at end of file
