@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007, 2008 XStream Committers.
+ * Copyright (C) 2007, 2008, 2009, 2011 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -10,21 +10,7 @@
  */
 package com.thoughtworks.xstream.mapper;
 
-import com.thoughtworks.xstream.InitializationException;
-import com.thoughtworks.xstream.XStream;
-import com.thoughtworks.xstream.annotations.XStreamAlias;
-import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
-import com.thoughtworks.xstream.annotations.XStreamConverter;
-import com.thoughtworks.xstream.annotations.XStreamConverters;
-import com.thoughtworks.xstream.annotations.XStreamImplicit;
-import com.thoughtworks.xstream.annotations.XStreamImplicitCollection;
-import com.thoughtworks.xstream.annotations.XStreamOmitField;
-import com.thoughtworks.xstream.converters.Converter;
-import com.thoughtworks.xstream.converters.ConverterRegistry;
-import com.thoughtworks.xstream.converters.reflection.ReflectionProvider;
-import com.thoughtworks.xstream.core.JVM;
-import com.thoughtworks.xstream.core.util.DependencyInjectionFactory;
-
+import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.lang.reflect.GenericArrayType;
 import java.lang.reflect.Modifier;
@@ -33,7 +19,7 @@ import java.lang.reflect.Type;
 import java.lang.reflect.TypeVariable;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
+import java.util.BitSet;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -41,7 +27,25 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.WeakHashMap;
+
+import com.thoughtworks.xstream.InitializationException;
+import com.thoughtworks.xstream.annotations.XStreamAlias;
+import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
+import com.thoughtworks.xstream.annotations.XStreamConverter;
+import com.thoughtworks.xstream.annotations.XStreamConverters;
+import com.thoughtworks.xstream.annotations.XStreamImplicit;
+import com.thoughtworks.xstream.annotations.XStreamImplicitCollection;
+import com.thoughtworks.xstream.annotations.XStreamInclude;
+import com.thoughtworks.xstream.annotations.XStreamOmitField;
+import com.thoughtworks.xstream.converters.Converter;
+import com.thoughtworks.xstream.converters.ConverterLookup;
+import com.thoughtworks.xstream.converters.ConverterMatcher;
+import com.thoughtworks.xstream.converters.ConverterRegistry;
+import com.thoughtworks.xstream.converters.SingleValueConverter;
+import com.thoughtworks.xstream.converters.SingleValueConverterWrapper;
+import com.thoughtworks.xstream.converters.reflection.ReflectionProvider;
+import com.thoughtworks.xstream.core.JVM;
+import com.thoughtworks.xstream.core.util.DependencyInjectionFactory;
 
 
 /**
@@ -61,8 +65,9 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
     private final FieldAliasingMapper fieldAliasingMapper;
     private final AttributeMapper attributeMapper;
     private final LocalConversionMapper localConversionMapper;
-    private final Map<Class<?>, Converter> converterCache = new HashMap<Class<?>, Converter>();
-    private final Set<Class<?>> annotatedTypes = new WeakHashSet<Class<?>>();
+    private final Map<Class<?>, Map<List<Object>, Converter>> converterCache = 
+            new HashMap<Class<?>, Map<List<Object>, Converter>>();
+    private final Set<Class<?>> annotatedTypes = new HashSet<Class<?>>();
 
     /**
      * Construct an AnnotationMapper.
@@ -71,7 +76,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
      * @since 1.3
      */
     public AnnotationMapper(
-        final Mapper wrapped, final ConverterRegistry converterRegistry,
+        final Mapper wrapped, final ConverterRegistry converterRegistry, final ConverterLookup converterLookup,
         final ClassLoader classLoader, final ReflectionProvider reflectionProvider,
         final JVM jvm) {
         super(wrapped);
@@ -84,11 +89,11 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
         attributeMapper = (AttributeMapper)lookupMapperOfType(AttributeMapper.class);
         localConversionMapper = (LocalConversionMapper)lookupMapperOfType(LocalConversionMapper.class);
         locked = true;
-        arguments = new Object[]{this, classLoader, reflectionProvider, jvm};
+        arguments = new Object[]{this, classLoader, reflectionProvider, jvm, converterLookup};
     }
 
     @Override
-    public String realMember(Class type, String serialized) {
+    public String realMember(final Class type, final String serialized) {
         if (!locked) {
             processAnnotations(type);
         }
@@ -96,7 +101,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
     }
 
     @Override
-    public String serializedClass(Class type) {
+    public String serializedClass(final Class type) {
         if (!locked) {
             processAnnotations(type);
         }
@@ -104,7 +109,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
     }
 
     @Override
-    public Class defaultImplementationOf(Class type) {
+    public Class defaultImplementationOf(final Class type) {
         if (!locked) {
             processAnnotations(type);
         }
@@ -123,7 +128,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
         return super.getLocalConverter(definedIn, fieldName);
     }
 
-    public void autodetectAnnotations(boolean mode) {
+    public void autodetectAnnotations(final boolean mode) {
         locked = !mode;
     }
 
@@ -134,7 +139,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
         locked = true;
         synchronized (annotatedTypes) {
             final Set<Class<?>> types = new UnprocessedTypesSet();
-            for (Class initialType : initialTypes) {
+            for (final Class initialType : initialTypes) {
                 types.add(initialType);
             }
             processTypes(types);
@@ -200,10 +205,10 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
 
     private void addParametrizedTypes(Type type, final Set<Class<?>> types) {
         final Set<Type> processedTypes = new HashSet<Type>();
-        Set<Type> localTypes = new LinkedHashSet<Type>() {
+        final Set<Type> localTypes = new LinkedHashSet<Type>() {
 
             @Override
-            public boolean add(Type o) {
+            public boolean add(final Type o) {
                 if (o instanceof Class) {
                     return types.add((Class<?>)o);
                 }
@@ -214,11 +219,11 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
         while (type != null) {
             processedTypes.add(type);
             if (type instanceof Class) {
-                Class<?> clazz = (Class<?>)type;
+                final Class<?> clazz = (Class<?>)type;
                 types.add(clazz);
                 if (!clazz.isPrimitive()) {
                     final TypeVariable<?>[] typeParameters = clazz.getTypeParameters();
-                    for (TypeVariable<?> typeVariable : typeParameters) {
+                    for (final TypeVariable<?> typeVariable : typeParameters) {
                         localTypes.add(typeVariable);
                     }
                     localTypes.add(clazz.getGenericSuperclass());
@@ -229,14 +234,14 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
             } else if (type instanceof TypeVariable) {
                 final TypeVariable<?> typeVariable = (TypeVariable<?>)type;
                 final Type[] bounds = typeVariable.getBounds();
-                for (Type bound : bounds) {
+                for (final Type bound : bounds) {
                     localTypes.add(bound);
                 }
             } else if (type instanceof ParameterizedType) {
                 final ParameterizedType parametrizedType = (ParameterizedType)type;
                 localTypes.add(parametrizedType.getRawType());
-                Type[] actualArguments = parametrizedType.getActualTypeArguments();
-                for (Type actualArgument : actualArguments) {
+                final Type[] actualArguments = parametrizedType.getActualTypeArguments();
+                for (final Type actualArgument : actualArguments) {
                     localTypes.add(actualArgument);
                 }
             } else if (type instanceof GenericArrayType) {
@@ -245,7 +250,7 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
             }
 
             if (!localTypes.isEmpty()) {
-                Iterator<Type> iter = localTypes.iterator();
+                final Iterator<Type> iter = localTypes.iterator();
                 type = iter.next();
                 iter.remove();
             } else {
@@ -258,7 +263,8 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
         if (converterRegistry != null) {
             final XStreamConverters convertersAnnotation = type
                 .getAnnotation(XStreamConverters.class);
-            final XStreamConverter converterAnnotation = type.getAnnotation(XStreamConverter.class);
+            final XStreamConverter converterAnnotation = type
+                .getAnnotation(XStreamConverter.class);
             final List<XStreamConverter> annotations = convertersAnnotation != null
                 ? new ArrayList<XStreamConverter>(Arrays.asList(convertersAnnotation.value()))
                 : new ArrayList<XStreamConverter>();
@@ -266,14 +272,14 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
                 annotations.add(converterAnnotation);
             }
             for (final XStreamConverter annotation : annotations) {
-                final Class<? extends Converter> converterType = annotation.value();
-                final Converter converter = cacheConverter(converterType);
+                final Converter converter = cacheConverter(
+                    annotation, converterAnnotation != null ? type : null);
                 if (converter != null) {
-                    if (converter != converterAnnotation || converter.canConvert(type)) {
-                        converterRegistry.registerConverter(converter, XStream.PRIORITY_NORMAL);
+                    if (converterAnnotation != null || converter.canConvert(type)) {
+                        converterRegistry.registerConverter(converter, annotation.priority());
                     } else {
                         throw new InitializationException("Converter "
-                            + converterType.getName()
+                            + annotation.value().getName()
                             + " cannot handle annotated class "
                             + type.getName());
                     }
@@ -353,8 +359,8 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
                     + FieldAliasingMapper.class.getName()
                     + " available");
             }
-            fieldAliasingMapper.addFieldAlias(aliasAnnotation.value(), field
-                .getDeclaringClass(), field.getName());
+            fieldAliasingMapper.addFieldAlias(
+                aliasAnnotation.value(), field.getDeclaringClass(), field.getName());
         }
     }
 
@@ -379,20 +385,35 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
                     + ImplicitCollectionMapper.class.getName()
                     + " available");
             }
-            String fieldName = field.getName();
-            String itemFieldName = implicitAnnotation.itemFieldName();
+            final String fieldName = field.getName();
+            final String itemFieldName = implicitAnnotation.itemFieldName();
+            final String keyFieldName = implicitAnnotation.keyFieldName();
+            boolean isMap = Map.class.isAssignableFrom(field.getType());
             Class itemType = null;
-            final Type genericType = field.getGenericType();
-            if (genericType instanceof ParameterizedType) {
-                final Type typeArgument = ((ParameterizedType)genericType)
-                    .getActualTypeArguments()[0];
-                itemType = getClass(typeArgument);
+            if (!field.getType().isArray()) {
+                final Type genericType = field.getGenericType();
+                if (genericType instanceof ParameterizedType) {
+                    final Type[] actualTypeArguments = ((ParameterizedType)genericType)
+                        .getActualTypeArguments();
+                    final Type typeArgument = actualTypeArguments[isMap ? 1 : 0];
+                    itemType = getClass(typeArgument);
+                }
             }
-            if (itemFieldName != null && !"".equals(itemFieldName)) {
+            if (isMap) {
                 implicitCollectionMapper.add(
-                    field.getDeclaringClass(), fieldName, itemFieldName, itemType);
+                    field.getDeclaringClass(), fieldName,
+                    itemFieldName != null && !"".equals(itemFieldName) ? itemFieldName : null,
+                    itemType, keyFieldName != null && !"".equals(keyFieldName)
+                        ? keyFieldName
+                        : null);
             } else {
-                implicitCollectionMapper.add(field.getDeclaringClass(), fieldName, itemType);
+                if (itemFieldName != null && !"".equals(itemFieldName)) {
+                    implicitCollectionMapper.add(
+                        field.getDeclaringClass(), fieldName, itemFieldName, itemType);
+                } else {
+                    implicitCollectionMapper
+                        .add(field.getDeclaringClass(), fieldName, itemType);
+                }
             }
         }
     }
@@ -413,33 +434,89 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
     private void processLocalConverterAnnotation(final Field field) {
         final XStreamConverter annotation = field.getAnnotation(XStreamConverter.class);
         if (annotation != null) {
-            final Class<? extends Converter> converterType = annotation.value();
-            final Converter converter = cacheConverter(converterType);
+            final Converter converter = cacheConverter(annotation, field.getType());
             if (converter != null) {
                 if (localConversionMapper == null) {
                     throw new InitializationException("No "
                         + LocalConversionMapper.class.getName()
                         + " available");
                 }
-                localConversionMapper.registerLocalConverter(field.getDeclaringClass(), field
-                    .getName(), converter);
+                localConversionMapper.registerLocalConverter(
+                    field.getDeclaringClass(), field.getName(), converter);
             }
         }
     }
 
-    private Converter cacheConverter(final Class<? extends Converter> converterType) {
-        Converter converter = converterCache.get(converterType);
-        if (converter == null) {
-            try {
-                converter = (Converter)DependencyInjectionFactory.newInstance(
-                    converterType, arguments);
-                converterCache.put(converterType, converter);
-            } catch (final Exception e) {
-                throw new InitializationException("Cannot instantiate converter "
-                    + converterType.getName(), e);
+    private Converter cacheConverter(final XStreamConverter annotation,
+        final Class targetType) {
+        Converter result = null;
+        final Object[] args;
+        final List<Object> parameter = new ArrayList<Object>();
+        if (targetType != null) {
+            parameter.add(targetType);
+        }
+        final List<Object> arrays = new ArrayList<Object>();
+        arrays.add(annotation.booleans());
+        arrays.add(annotation.bytes());
+        arrays.add(annotation.chars());
+        arrays.add(annotation.doubles());
+        arrays.add(annotation.floats());
+        arrays.add(annotation.ints());
+        arrays.add(annotation.longs());
+        arrays.add(annotation.shorts());
+        arrays.add(annotation.strings());
+        arrays.add(annotation.types());
+        for(Object array : arrays) {
+            if (array != null) {
+                int length = Array.getLength(array);
+                for (int i = 0; i < length; i++ ) {
+                    Object object = Array.get(array, i);
+                    if (!parameter.contains(object)) {
+                        parameter.add(object);
+                    }
+                }
             }
         }
-        return converter;
+        final Class<? extends ConverterMatcher> converterType = annotation.value();
+        Map<List<Object>, Converter> converterMapping = converterCache.get(converterType);
+        if (converterMapping != null) {
+            result = converterMapping.get(parameter);
+        }
+        if (result == null) {
+            int size = parameter.size();
+            if (size > 0) {
+                args = new Object[arguments.length + size];
+                System.arraycopy(arguments, 0, args, size, arguments.length);
+                System.arraycopy(parameter.toArray(new Object[size]), 0, args, 0, size);
+            } else {
+                args = arguments;
+            }
+
+            final BitSet usedArgs = new BitSet();
+            final Converter converter;
+            try {
+                if (SingleValueConverter.class.isAssignableFrom(converterType)
+                    && !Converter.class.isAssignableFrom(converterType)) {
+                    final SingleValueConverter svc = (SingleValueConverter)DependencyInjectionFactory
+                        .newInstance(converterType, args, usedArgs);
+                    converter = new SingleValueConverterWrapper(svc);
+                } else {
+                    converter = (Converter)DependencyInjectionFactory.newInstance(
+                        converterType, args, usedArgs);
+                }
+            } catch (final Exception e) {
+                throw new InitializationException("Cannot instantiate converter "
+                    + converterType.getName()
+                    + (targetType != null ? " for type " + targetType.getName() : ""), e);
+            }
+            if (converterMapping == null) {
+                converterMapping = new HashMap<List<Object>, Converter>();
+                converterCache.put(converterType, converterMapping);
+            }
+            converterMapping.put(parameter, converter);
+            result = converter;
+        }
+        return result;
     }
 
     private Class<?> getClass(final Type typeArgument) {
@@ -462,85 +539,22 @@ public class AnnotationMapper extends MapperWrapper implements AnnotationConfigu
                 type = type.getComponentType();
             }
             final String name = type.getName();
-            if (name.startsWith("java.") || name.startsWith("java.")) {
+            if (name.startsWith("java.") || name.startsWith("javax.")) {
                 return false;
             }
-            return annotatedTypes.contains(type) ? false : super.add(type);
-        }
-    }
-
-    private static class WeakHashSet<K> implements Set<K> {
-
-        private static Object NULL = new Object();
-        private WeakHashMap<K, Object> map = new WeakHashMap<K, Object>();
-
-        public boolean add(K o) {
-            return map.put(o, NULL) == null;
-        }
-
-        public boolean addAll(Collection<? extends K> c) {
-            boolean ret = false;
-            for (K k : c) {
-                ret = add(k) | false;
-            }
-            return ret;
-        }
-
-        public void clear() {
-            map.clear();
-        }
-
-        public boolean contains(Object o) {
-            return map.containsKey(o);
-        }
-
-        public boolean containsAll(Collection<?> c) {
-            return map.keySet().containsAll(c);
-        }
-
-        public boolean isEmpty() {
-            return map.isEmpty();
-        }
-
-        public Iterator<K> iterator() {
-            return map.keySet().iterator();
-        }
-
-        public boolean remove(Object o) {
-            return map.remove(o) != null;
-        }
-
-        public boolean removeAll(Collection<?> c) {
-            boolean ret = false;
-            for (Object object : c) {
-                ret = remove(object) | false;
-            }
-            return ret;
-        }
-
-        public boolean retainAll(Collection<?> c) {
-            boolean ret = false;
-            for (final Iterator<K> iter = iterator(); iter.hasNext();) {
-                final K element = iter.next();
-                if (!c.contains(element)) {
-                    iter.remove();
-                    ret = true;
+            final boolean ret = annotatedTypes.contains(type) ? false : super.add(type);
+            if (ret) {
+                final XStreamInclude inc = type.getAnnotation(XStreamInclude.class);
+                if (inc != null) {
+                    final Class<?>[] incTypes = inc.value();
+                    if (incTypes != null) {
+                        for (final Class<?> incType : incTypes) {
+                            add(incType);
+                        }
+                    }
                 }
             }
             return ret;
         }
-
-        public int size() {
-            return map.size();
-        }
-
-        public Object[] toArray() {
-            return map.keySet().toArray();
-        }
-
-        public <T> T[] toArray(T[] a) {
-            return map.keySet().toArray(a);
-        }
-
     }
 }
