@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2004, 2005 Joe Walnes.
- * Copyright (C) 2006, 2007 XStream Committers.
+ * Copyright (C) 2006, 2007, 2008, 2009, 2011 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -11,7 +11,6 @@
  */
 package com.thoughtworks.acceptance;
 
-import com.thoughtworks.acceptance.objects.SampleLists;
 import com.thoughtworks.acceptance.objects.StandardObject;
 import com.thoughtworks.xstream.InitializationException;
 
@@ -58,7 +57,6 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
         xstream.alias("room", Room.class);
         xstream.alias("house", House.class);
         xstream.alias("person", Person.class);
-        xstream.alias("sample", SampleLists.class);
     }
 
     public void testWithout() {
@@ -103,6 +101,7 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
     }
 
     public static class MegaFarm extends Farm {
+        List names;
         public MegaFarm(int size) {
             super(size);
         }
@@ -130,6 +129,34 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
         assertBothWays(farm, expected);
     }
 
+    public void testSupportsInheritedAndDirectDeclaredImplicitCollectionAtOnce() {
+        xstream.alias("MEGA-farm", MegaFarm.class);
+
+        MegaFarm farm = new MegaFarm(100); // subclass
+        farm.add(new Animal("Cow"));
+        farm.add(new Animal("Sheep"));
+        farm.names = new ArrayList();
+        farm.names.add("McDonald");
+        farm.names.add("Ponte Rosa");
+        
+        String expected = "" +
+                "<MEGA-farm>\n" +
+                "  <size>100</size>\n" +
+                "  <animal>\n" +
+                "    <name>Cow</name>\n" +
+                "  </animal>\n" +
+                "  <animal>\n" +
+                "    <name>Sheep</name>\n" +
+                "  </animal>\n" +
+                "  <name>McDonald</name>\n" +
+                "  <name>Ponte Rosa</name>\n" +
+                "</MEGA-farm>";
+
+        xstream.addImplicitCollection(Farm.class, "animals");
+        xstream.addImplicitCollection(MegaFarm.class, "names", "name", String.class);
+        assertBothWays(farm, expected);
+    }
+
     public void testAllowsSubclassToOverrideImplicitCollectionInSuperclass() {
         xstream.alias("MEGA-farm", MegaFarm.class);
 
@@ -237,9 +264,24 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
         assertEquals(house.getPeople(), serializedHouse.getPeople());
         assertEquals(house.getRooms(), serializedHouse.getRooms());
     }
+    
+    public void testWithEMPTY_LIST() {
+        House house = new House();
+        house.people = Collections.EMPTY_LIST;
+        house.rooms = Collections.EMPTY_LIST;
+        xstream.addImplicitCollection(House.class, "rooms", Room.class);
+        xstream.addImplicitCollection(House.class, "people", Person.class);
+        assertEquals("<house/>", xstream.toXML(house));
+    }
 
     public static class Zoo extends StandardObject {
-        private Set animals = new HashSet();
+        private Set animals;
+        public Zoo() {
+            this(new HashSet());
+        }
+        public Zoo(Set set) {
+            animals = set;
+        }
         public void add(Animal animal) {
             animals.add(animal);
         }
@@ -281,6 +323,26 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
         assertTrue("Collection was a " + zoo.animals.getClass().getName(), zoo.animals instanceof TreeSet);
     }
 
+    public void testWithSortedSet() {
+        Zoo zoo = new Zoo(new TreeSet());
+        zoo.add(new Animal("Lion"));
+        zoo.add(new Animal("Ape"));
+
+        String expected = "" +
+                "<zoo>\n" +
+                "  <animal>\n" +
+                "    <name>Ape</name>\n" +
+                "  </animal>\n" +
+                "  <animal>\n" +
+                "    <name>Lion</name>\n" +
+                "  </animal>\n" +
+                "</zoo>";
+
+        xstream.addImplicitCollection(Zoo.class, "animals");
+        xstream.addDefaultImplementation(TreeSet.class, Set.class);
+        assertBothWays(zoo, expected);
+    }
+
     public static class Aquarium extends StandardObject {
         private String name;
         private List fish = new ArrayList();
@@ -314,6 +376,27 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
         assertBothWays(aquarium, expected);
     }
     
+    public void testWithImplicitNameMatchingTheNameOfTheFieldWithTheCollection() {
+        Aquarium aquarium = new Aquarium("hatchery");
+        aquarium.addFish("salmon");
+        aquarium.addFish("halibut");
+        aquarium.addFish("snapper");
+
+        String expected = "" +
+                "<aquarium>\n" +
+                "  <name>hatchery</name>\n" +
+                "  <fish>salmon</fish>\n" +
+                "  <fish>halibut</fish>\n" +
+                "  <fish>snapper</fish>\n" +
+                "</aquarium>";
+
+        xstream.alias("aquarium", Aquarium.class);
+        xstream.alias("fish", String.class);
+        xstream.addImplicitCollection(Aquarium.class, "fish");
+
+        assertBothWays(aquarium, expected);
+    }
+    
     public void testWithAliasedItemNameMatchingTheAliasedNameOfTheFieldWithTheCollection() {
         Aquarium aquarium = new Aquarium("hatchery");
         aquarium.addFish("salmon");
@@ -343,4 +426,40 @@ public class ImplicitCollectionTest extends AbstractAcceptanceTest {
             assertTrue(e.getMessage().indexOf("declares no collection") >= 0);
         }
     }
+
+    public void testWithNullElement() {
+        Farm farm = new Farm(100);
+        farm.add(null);
+        farm.add(new Animal("Cow"));
+
+        String expected = "" +
+                "<farm>\n" +
+                "  <size>100</size>\n" +
+                "  <null/>\n" +
+                "  <animal>\n" +
+                "    <name>Cow</name>\n" +
+                "  </animal>\n" +
+                "</farm>";
+
+        xstream.addImplicitCollection(Farm.class, "animals");
+        assertBothWays(farm, expected);
+    }
+
+    public void testWithAliasAndNullElement() {
+        Farm farm = new Farm(100);
+        farm.add(null);
+        farm.add(new Animal("Cow"));
+
+        String expected = "" +
+                "<farm>\n" +
+                "  <size>100</size>\n" +
+                "  <null/>\n" +
+                "  <beast>\n" +
+                "    <name>Cow</name>\n" +
+                "  </beast>\n" +
+                "</farm>";
+
+        xstream.addImplicitCollection(Farm.class, "animals", "beast", Animal.class);
+        assertBothWays(farm, expected);
+    }
 }
