@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2005 Joe Walnes.
- * Copyright (C) 2006, 2007 XStream Committers.
+ * Copyright (C) 2006, 2007, 2010 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -48,7 +48,7 @@ public class OmitFieldsTest extends AbstractAcceptanceTest {
         assertEquals("c", out.neverIgnore);
     }
 
-    public void testAdditionalFieldsCanBeExplicitlyOmittedThroughFacade() {
+    public void testAdditionalFieldsCanBeExplicitlyOmitted() {
         Thing in = new Thing();
         in.alwaysIgnore = "a";
         in.sometimesIgnore = "b";
@@ -165,6 +165,38 @@ public class OmitFieldsTest extends AbstractAcceptanceTest {
         assertEquals("c", out.neverIgnore);
     }
 
+    public void testDeletedElementWithReferenceCanBeOmitted() {
+        String expectedXml = ""
+            + "<thing>\n"
+            + "  <meanwhileDeletedIgnore reference=\"..\"/>\n"
+            + "  <sometimesIgnore>b</sometimesIgnore>\n"
+            + "  <neverIgnore>c</neverIgnore>\n"
+            + "</thing>";
+
+        xstream.alias("thing", Thing.class);
+        xstream.omitField(Thing.class, "meanwhileDeletedIgnore");
+
+        Thing out = (Thing)xstream.fromXML(expectedXml);
+        assertEquals("b", out.sometimesIgnore);
+        assertEquals("c", out.neverIgnore);
+    }
+
+    public void testDeletedElementWithClassAttributeCanBeOmitted() {
+        String expectedXml = ""
+            + "<thing>\n"
+            + "  <meanwhileDeletedIgnore class=\"thing\"/>\n"
+            + "  <sometimesIgnore>b</sometimesIgnore>\n"
+            + "  <neverIgnore>c</neverIgnore>\n"
+            + "</thing>";
+
+        xstream.alias("thing", Thing.class);
+        xstream.omitField(Thing.class, "meanwhileDeletedIgnore");
+
+        Thing out = (Thing)xstream.fromXML(expectedXml);
+        assertEquals("b", out.sometimesIgnore);
+        assertEquals("c", out.neverIgnore);
+    }
+
     public void testDeletedAttributeCanBeOmitted() {
         String expectedXml = ""
             + "<thing meanwhileDeletedIgnore='c'>\n"
@@ -198,6 +230,22 @@ public class OmitFieldsTest extends AbstractAcceptanceTest {
         assertEquals("c", out.neverIgnore);
     }
 
+    public void testExistingFieldsCanBeExplicitlyOmittedAtDeserialization() {
+        String actualXml = ""
+            + "<thing>\n"
+            + "  <sometimesIgnore>foo</sometimesIgnore>\n" 
+            + "  <neverIgnore>c</neverIgnore>\n" 
+            + "</thing>";
+
+        xstream.alias("thing", Thing.class);
+        xstream.omitField(Thing.class, "sometimesIgnore");
+
+        Thing out = (Thing)xstream.fromXML(actualXml);
+        assertEquals(null, out.alwaysIgnore);
+        assertEquals(null, out.sometimesIgnore);
+        assertEquals("c", out.neverIgnore);
+    }
+
     static class ThingAgain extends Thing {
         String sometimesIgnore;
 
