@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006 Joe Walnes.
- * Copyright (C) 2006, 2007, 2008 XStream Committers.
+ * Copyright (C) 2006, 2007, 2008, 2009, 2011 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -28,7 +28,12 @@ import java.util.Set;
 import java.util.TimeZone;
 import java.util.TreeMap;
 
+import com.thoughtworks.acceptance.objects.Original;
+import com.thoughtworks.acceptance.objects.OwnerOfExternalizable;
+import com.thoughtworks.acceptance.objects.Replaced;
+import com.thoughtworks.acceptance.objects.SomethingExternalizable;
 import com.thoughtworks.xstream.XStream;
+import com.thoughtworks.xstream.core.JVM;
 
 import junit.framework.TestCase;
 
@@ -41,14 +46,22 @@ import junit.framework.TestCase;
  * @author J&ouml;rg Schaible
  */
 public class JsonHierarchicalStreamDriverTest extends TestCase {
-    private XStream xstream;
+    protected XStream xstream;
 
     /**
      * @see junit.framework.TestCase#setUp()
      */
     protected void setUp() throws Exception {
         super.setUp();
-        xstream = new XStream(new JsonHierarchicalStreamDriver());
+        xstream = new XStream(createDriver());
+    }
+    
+    protected JsonHierarchicalStreamDriver createDriver() {
+        return new JsonHierarchicalStreamDriver();
+    }
+    
+    protected String normalizeExpectation(String expected) {
+        return expected.replace('\'', '"');
     }
 
     public void testDoesNotSupportReader() {
@@ -71,7 +84,8 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
     public void testCanMarshalSimpleTypes() {
 
-        String expected = ("{'innerMessage': {\n"
+        String expected = normalizeExpectation(""
+            + "{'innerMessage': {\n"
             + "  'long1': 5,\n"
             + "  'long2': 42,\n"
             + "  'greeting': 'hello',\n"
@@ -88,7 +102,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
             + "    'bool': false,\n"
             + "    'char1': '\\u0000'\n"
             + "  }\n"
-            + "}}").replace('\'', '"');
+            + "}}");
 
         xstream.alias("innerMessage", Message.class);
 
@@ -127,36 +141,37 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         }
     }
 
-    String expectedMenuStart = ""
+    protected String expectedMenuStart = ""
         + "{'menu': {\n"
         + "  'id': 'file',\n"
         + "  'value': 'File:',\n"
         + "  'popup': {\n"
         + "    'menuitem': [";
-    String expectedNew = ""
+    protected String expectedNew = ""
         + "      {\n"
         + "        'value': 'New',\n"
         + "        'onclick': 'CreateNewDoc()'\n"
         + "      }";
-    String expectedOpen = ""
+    protected String expectedOpen = ""
         + "      {\n"
         + "        'value': 'Open',\n"
         + "        'onclick': 'OpenDoc()'\n"
         + "      }";
-    String expectedClose = ""
+    protected String expectedClose = ""
         + "      {\n"
         + "        'value': 'Close',\n"
         + "        'onclick': 'CloseDoc()'\n"
         + "      }";
-    String expectedMenuEnd = "" + "    ]\n" + "  }\n" + "}}";
-    String expected = (expectedMenuStart
+    protected String expectedMenuEnd = "" + "    ]\n" + "  }\n" + "}}";
+    protected String expected = "" //
+        + expectedMenuStart
         + "\n"
         + expectedNew
         + ",\n"
         + expectedOpen
         + ",\n"
         + expectedClose
-        + "\n" + expectedMenuEnd).replace('\'', '"');
+        + "\n" + expectedMenuEnd;
 
     public void testCanMarshalLists() {
 
@@ -167,7 +182,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
         MenuWithList menu = new MenuWithList();
 
-        assertEquals(expected, xstream.toXML(menu));
+        assertEquals(normalizeExpectation(expected), xstream.toXML(menu));
     }
 
     public void testCanMarshalArrays() {
@@ -177,7 +192,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
         MenuWithArray menu = new MenuWithArray();
 
-        assertEquals(expected, xstream.toXML(menu));
+        assertEquals(normalizeExpectation(expected), xstream.toXML(menu));
     }
 
     public void testCanMarshalSets() {
@@ -190,7 +205,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         MenuWithSet menu = new MenuWithSet();
 
         String json = xstream.toXML(menu);
-        assertTrue(json.startsWith(expectedMenuStart.replace('\'', '"')));
+        assertTrue(json.startsWith(normalizeExpectation(expectedMenuStart)));
         assertTrue(json.indexOf(expectedNew.replace('\'', '"')) > 0);
         assertTrue(json.indexOf(expectedOpen.replace('\'', '"')) > 0);
         assertTrue(json.indexOf(expectedClose.replace('\'', '"')) > 0);
@@ -261,7 +276,8 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
         // This also from http://www.expected.org/example.html
 
-        String expected = ("{'widget': {\n"
+        String expected = normalizeExpectation("" // 
+            + "{'widget': {\n"
             + "  'debug': 'on',\n"
             + "  'window': {\n"
             + "    'title': 'Sample Konfabulator Widget',\n"
@@ -286,7 +302,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
             + "    'alignment': 'center',\n"
             + "    'onMouseUp': 'sun1.opacity = (sun1.opacity / 100) * 90;'\n"
             + "  }\n"
-            + "}}").replace('\'', '"');
+            + "}}");
 
         xstream.alias("widget", Widget.class);
         xstream.alias("window", Window.class);
@@ -333,30 +349,49 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
     }
 
     public void testColor() {
-        Color color = Color.black;
-        String expected = ("{'awt-color': {\n"
-            + "  'red': 0,\n"
-            + "  'green': 0,\n"
-            + "  'blue': 0,\n"
-            + "  'alpha': 255\n"
-            + "}}").replace('\'', '"');
-        assertEquals(expected, xstream.toXML(color));
+        boolean isHeadless = Boolean.valueOf(System.getProperty("java.awt.headless", "false")).booleanValue();
+        if (!isHeadless || JVM.is15()) {
+            Color color = Color.black;
+            String expected = normalizeExpectation("" // 
+                + "{'awt-color': {\n"
+                + "  'red': 0,\n"
+                + "  'green': 0,\n"
+                + "  'blue': 0,\n"
+                + "  'alpha': 255\n"
+                + "}}");
+            assertEquals(expected, xstream.toXML(color));
+        }
     }
 
     public void testDoesHandleQuotesAndEscapes() {
         String[] strings = new String[]{
             "last\"", "\"first", "\"between\"", "around \"\" it", "back\\slash",};
-        String expected = (""
-            + "{#string-array#: [\n"
-            + "  #last\\\"#,\n"
-            + "  #\\\"first#,\n"
-            + "  #\\\"between\\\"#,\n"
-            + "  #around \\\"\\\" it#,\n"
-            + "  #back\\\\slash#\n"
-            + "]}").replace('#', '"');
+        String expected = normalizeExpectation(""
+            + "{'string-array': [\n"
+            + "  'last\\\"',\n"
+            + "  '\\\"first',\n"
+            + "  '\\\"between\\\"',\n"
+            + "  'around \\\"\\\" it',\n"
+            + "  'back\\\\slash'\n"
+            + "]}");
         assertEquals(expected, xstream.toXML(strings));
     }
 
+    public void testDoesEscapeValuesAccordingRfc4627() {
+        String expected = normalizeExpectation("{'string': '\\u0000\\u0001\\u001f \uffee'}");
+        assertEquals(expected, xstream.toXML("\u0000\u0001\u001f\u0020\uffee"));
+    }
+
+    public void testSimpleInteger() {
+        String expected = normalizeExpectation("{'int': 123}");
+        assertEquals(expected, xstream.toXML(new Integer(123)));
+    }
+
+    public void testBracesAndSquareBracketsAreNotEscaped() {
+        String expected = normalizeExpectation("{'string': '..{}[],,'}");
+        assertEquals(expected, xstream.toXML("..{}[],,"));
+    }
+
     public void testCanMarshalSimpleTypesWithNullMembers() {
         Msg message = new Msg("hello");
         Msg message2 = new Msg(null);
@@ -364,11 +399,11 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
         xstream.alias("innerMessage", Msg.class);
 
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'innerMessage': {\n"
             + "  'greeting': 'hello',\n"
             + "  'innerMessage': {}\n"
-            + "}}").replace('\'', '"');
+            + "}}");
         assertEquals(expected, xstream.toXML(message));
     }
 
@@ -384,8 +419,10 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
     public void testCanMarshalElementWithEmptyArray() {
         xstream.alias("element", ElementWithEmptyArray.class);
 
-        String expected = ("" + "{'element': {\n" + "  'array': [\n" + "  ]\n" + "}}").replace(
-            '\'', '"');
+        String expected = normalizeExpectation("" // 
+            + "{'element': {\n"
+            + "  'array': []\n"
+            + "}}");
         assertEquals(expected, xstream.toXML(new ElementWithEmptyArray()));
     }
 
@@ -412,11 +449,11 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         int idx1 = actual.indexOf("one");
         int idx2 = actual.indexOf("two");
 
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'map': [\n"
             + ((idx1 < idx2 ? entry1 : entry2) + ",\n")
             + ((idx1 < idx2 ? entry2 : entry1) + "\n") // no comma
-        + "]}").replace('\'', '"');
+        + "]}");
         assertEquals(expected, actual);
     }
 
@@ -439,11 +476,11 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         int idx1 = actual.indexOf("one");
         int idx2 = actual.indexOf("two");
 
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'properties': [\n"
             + ((idx1 < idx2 ? entry1 : entry2) + ",\n")
             + ((idx1 < idx2 ? entry2 : entry1) + "\n") // no comma
-        + "]}").replace('\'', '"');
+        + "]}");
         assertEquals(expected, actual);
     }
 
@@ -471,13 +508,13 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         int idx1 = actual.indexOf("one");
         int idx2 = actual.indexOf("two");
 
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'holder': {\n"
             + "  'map': [\n"
             + ((idx1 < idx2 ? entry1 : entry2) + ",\n")
             + ((idx1 < idx2 ? entry2 : entry1) + "\n")
             + "  ]\n" // no comma
-        + "}}").replace('\'', '"');
+        + "}}");
         assertEquals(expected, actual);
     }
 
@@ -487,13 +524,13 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
     public void testIgnoresAttributeForCollectionMember() {
         xstream.alias("keeper", CollectionKeeper.class);
-        String expected = ("" //
+        String expected = normalizeExpectation("" //
             + "{'keeper': {\n"
             + "  'coll': [\n"
             + "    'one',\n"
             + "    'two'\n"
             + "  ]\n"
-            + "}}").replace('\'', '"');
+            + "}}");
 
         final CollectionKeeper holder = new CollectionKeeper();
         holder.coll.add("one");
@@ -506,12 +543,12 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         xstream.alias("window", Window.class);
         xstream.useAttributeFor("width", int.class);
         xstream.useAttributeFor("height", int.class);
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'window': {\n"
             + "  '@width': '500',\n"
             + "  '@height': '500',\n"
             + "  'title': 'JUnit'\n"
-            + "}}").replace('\'', '"');
+            + "}}");
 
         final Window window = new Window();
         window.title = "JUnit";
@@ -528,7 +565,7 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
     public void testCanWriteEmbeddedCalendar() {
         xstream.alias("person", Person.class);
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'list': [\n"
             + "  {\n"
             + "    'firstName': 'Joe',\n"
@@ -538,14 +575,13 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
             + "      'timezone': 'Europe/London'\n"
             + "    },\n"
             + "    'titles': [\n"
-            + "      {},\n" // no-comparator element
             + "      [\n"
             + "        '1',\n"
             + "        'Mr'\n"
             + "      ]\n"
             + "    ]\n"
             + "  }\n"
-            + "]}").replace('\'', '"');
+            + "]}");
 
         Person person = new Person();
         person.firstName = "Joe";
@@ -559,11 +595,6 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
         assertEquals(expected, xstream.toXML(list));
     }
 
-    public void testDoesEscapeValuesAccordingRfc4627() {
-        String expected = "{'string': '\\u0000\\u0001\\u001f \uffee'}".replace('\'', '"');
-        assertEquals(expected, xstream.toXML("\u0000\u0001\u001f\u0020\uffee"));
-    }
-
     static class SingleValue {
         long l;
         URL url;
@@ -571,15 +602,80 @@ public class JsonHierarchicalStreamDriverTest extends TestCase {
 
     public void testSupportsAllConvertersWithASingleValue() throws MalformedURLException {
         xstream.alias("sv", SingleValue.class);
-        String expected = (""
+        String expected = normalizeExpectation(""
             + "{'sv': {\n"
             + "  'l': 4711,\n"
             + "  'url': 'http://localhost:8888'\n"
-            + "}}").replace('\'', '"');
+            + "}}");
 
         SingleValue value = new SingleValue();
         value.l = 4711;
         value.url = new URL("http://localhost:8888");
         assertEquals(expected, xstream.toXML(value));
     }
+    
+    static class SystemAttributes {
+        String name;
+        Object object;
+        Original original;
+    }
+    
+    public void testWillWriteTagValueAsDefaultValueIfNecessary() {
+        xstream.alias("sa", SystemAttributes.class);
+        xstream.alias("original", Original.class);
+        xstream.alias("replaced", Replaced.class);
+
+        SystemAttributes sa = new SystemAttributes();
+        sa.name = "joe";
+        sa.object = "walnes";
+        sa.original = new Original("hello world");
+
+        String expected = normalizeExpectation(""
+                + "{'sa': {\n"
+                + "  'name': 'joe',\n"
+                + "  'object': {\n"
+                + "    '@class': 'string',\n"
+                + "    '$': 'walnes'\n" 
+                +"  },\n"
+                + "  'original': {\n"
+                + "    '@resolves-to': 'replaced',\n"
+                + "    'replacedValue': 'HELLO WORLD'\n"
+                + "  }\n"
+                + "}}");
+
+        assertEquals(expected, xstream.toXML(sa));
+    }
+
+    public void testCanMarshalExternalizable() {
+        xstream.alias("ext", SomethingExternalizable.class);
+        
+        SomethingExternalizable in = new SomethingExternalizable("Joe", "Walnes");
+        String expected = normalizeExpectation(""
+            + "{'ext': [\n"
+            + "  3,\n"
+            + "  'JoeWalnes',\n"
+            + "  {},\n"
+            + "  'XStream'\n"
+            + "]}");
+
+        assertEquals(expected, xstream.toXML(in));
+    }
+
+    public void testCanMarshalEmbeddedExternalizable() {
+        xstream.alias("owner", OwnerOfExternalizable.class);
+        
+        OwnerOfExternalizable in = new OwnerOfExternalizable();
+        in.target = new SomethingExternalizable("Joe", "Walnes");
+        String expected = normalizeExpectation(""
+            + "{'owner': {\n"
+            + "  'target': [\n"
+            + "    3,\n"
+            + "    'JoeWalnes',\n"
+            + "    {},\n"
+            + "    'XStream'\n"
+            + "  ]\n"
+            + "}}");
+
+        assertEquals(expected, xstream.toXML(in));
+    }
 }
